-- [[file:../../arcology-feedgen.org::*Lua Filter][Lua Filter:1]]
local utils = require 'pandoc.utils'

local entries = {}
local keywords = {}
local variables = {}

set_meta_from_raw = function (raw)
   -- Don't do anything unless the block contains *org* markup.
   if raw.format ~= 'org' then return nil end

   -- extract variable name and value
   local name, value = raw.text:match '#%+(%w+):%s*(.+)$'
   if name and value then
      variables[name] = value
   end
end

-- thanks random github users https://gist.github.com/zwh8800/9b0442efadc97408ffff248bc8573064
local epoch = os.time{year=1970, month=1, day=1, hour=0}
function parse_org_date(org_date)
   local year, month, day, hour, minute = org_date:match("<?(%d+)%-(%d+)%-(%d+)%s%a+%s(%d+)%:(%d+)>?")
   local timestamp = os.time{year = year, month = month, day = day, hour = hour, min = minute, sec = 0} - epoch
   return timestamp
end
rfc3339ish = "%Y-%m-%dT%TZ"

set_entries_and_date = function(m)
   for name, value in pairs(variables) do
      m[name] = value
   end
   if m["FILETAGS"] ~= nil then
      kw_str = utils.stringify(m["FILETAGS"])
      kw_str:gsub("([^:]*)", function(tag)
                     if tag ~= "" then
                        table.insert(keywords, tag)
                     end
      end)
   end
   if m["keywords"] ~= nil then
      kw_str = utils.stringify(m["keywords"])
      kw_str:gsub("([^, ]*)", function(tag)
                     if tag ~= "" then
                        table.insert(keywords, tag)
                     end
      end)
   end
   if m.date == nil then
      m.date = os.date(rfc3339ish) --current time in iso8601/rfc3339
   end
   m.entries = entries
   m.keywords = keywords
   return m
end

extract_entries = function (blocks)
   pandoc.utils.make_sections(true, nil, blocks):walk
   {
      Div = function (div)
         if div.attributes.pubdate then
            local header = div.content[1]
            header.attributes.pubdate = nil
            header.attributes.number = nil
            div.attributes.number = nil
            header_tags = {}

            title_subbed = utils.stringify(header.content)
            for k,v in pairs(header.content) do
               if v.tag == "Span" and v.attributes["tag-name"] ~= nil then
                  table.insert(header_tags, v.attributes["tag-name"])
               end
            end

            entry = {
               content=div,
               title=title_subbed,
               rel=div.attributes.id,
               pubdate=os.date(rfc3339ish, parse_org_date(div.attributes.pubdate))
            }
            if #header_tags > 0 then
               entry["categories"] = header_tags
            end
            table.insert(entries, entry)
         end
      end
   }
end

return {
   {
      RawBlock = set_meta_from_raw,
      Blocks = extract_entries,
      Meta = set_entries_and_date
   }
}
-- Lua Filter:1 ends here
