# [[file:../../arcology-router.org::*Arcology Domain-Aware Routing][Arcology Domain-Aware Routing:1]]
from fastapi import FastAPI, Request
from sqlmodel import Session
from fastapi.responses import HTMLResponse, FileResponse, Response

from arcology.routing.util import render_page_from_key, templates, crawlable_pages_for_site
from arcology.feeds import render_feed_from_file
from arcology.arroyo import engine
from arcology.key import ArcologyKey
from arcology.arroyo import Keyword
from arcology.sigma import sigma


def decorate_app(app: FastAPI) -> FastAPI:
  import logging
  @app.get("/robots.txt", response_class=FileResponse, name="robots-txt")
  async def robots_txt(request: Request):
    with Session(engine) as session:
      key = ArcologyKey.from_request(request).site_key
      crawlable_pages = crawlable_pages_for_site(key, session)
      return templates.TemplateResponse("robots.txt.j2", dict(
          request=request,
          pages=crawlable_pages,
      ))

  @app.get("/sitemap/", response_class=HTMLResponse, name="sitemap_page")
  async def sitemap(request: Request):
    site = ArcologyKey.from_request(request).site
    return templates.TemplateResponse("sitemap.html.j2", dict(
      request=request,
      site=site,
    ))
  
  @app.get("/sitemap/json", name="sitemap_json")
  async def sitemap(request: Request):
    return sigma(engine)

  from arcology.arroyo import Feed, engine
  from sqlmodel import select, SQLModel, Session
  from typing import List
  
  def get_feeds(session: Session) -> List[Feed]:
    q = select(Feed)
    return session.exec(q).all()
  
  @app.get("/feeds.json", response_model=List[Feed], name="feed-list")
  async def feed_list(request: Request):
    with Session(engine) as session:
      feeds = get_feeds(session)
      return feeds
  @app.get("/{sub_key:path}.xml", response_class=Response, name="feed-route")
  async def feed_route(request: Request, sub_key: str):
    sub_key += ".xml" # dark laughter
    return Response(content=(await public_router(request, sub_key)), media_type="application/atom+xml")

  @app.get("/{sub_key:path}", response_class=HTMLResponse, name="base-route")
  @app.get("/{sub_key:path}/", response_class=HTMLResponse, name="base-route")
  async def public_router(request: Request, sub_key: str):
    key = None
  
    if request.headers.get('host',"").startswith('localhost'):
      key = ArcologyKey(key=sub_key)
    else:
      key = ArcologyKey.from_request(request)
  
    with Session(engine) as session:
      kw = Keyword.get("ARCOLOGY_FEED", key.key, session)
      if kw is not None:
        return await render_feed_from_file(request, kw.filename(), engine, key.site)
  
    return await render_page_from_key(request, key.key, engine, key.site)

  return app
# Arcology Domain-Aware Routing:1 ends here
